// Library file READFILE.C (used to build LIBWAV.A)
// Written by:
// Daniel W. Lewis (dlewis@scu.edu)
// Computer Engineering Department
// Santa Clara University
// Revised Jan 2, 2015

#include "libwav.h"

PRIVATE FMT_INFO *	ReadFormatChunk(FILE *fp, char *filespec) ;
PRIVATE AUDIO *		ReadDataChunk(FILE *fp, char *filespec, unsigned chunk_size) ;

AUDIO *ReadWAV16(char *filespec)
	{
	long int feof, fpos ;
	CHUNK_HDR chunk_hdr ;
	RIFF_INFO riff ;
	FMT_INFO *fmt_info ;
	AUDIO *audio ;
	FILE *fp ;

	if ((fp = fopen(filespec, "rb")) == NULL)
		_Error("Error opening file %s", filespec) ;

	if (fread(&chunk_hdr, sizeof(CHUNK_HDR), 1, fp) != 1)
		_Error("Can't read RIFF chunk header of file %s", filespec) ;
	if (chunk_hdr.id != 0x46464952)	// 'RIFF'
		_Error("File %s contains incorrect 'RIFF' ID (0x%08X)", filespec, chunk_hdr.id) ;
	feof = sizeof(CHUNK_HDR) + chunk_hdr.size ;

	if (fread(&riff, sizeof(RIFF_INFO), 1, fp) != 1)
		_Error("Can't read RIFF format of file %s", filespec) ;
	if (riff.format != 0x45564157)	// 'WAVE'
		_Error("File %s contains incorrect 'WAVE' ID (0x%08X)", filespec, riff.format) ;

	// find the format and data chunks ...
	fmt_info = NULL ;
	audio = NULL ;
	while ((fpos = ftell(fp)) < feof)
		{
		if (fread(&chunk_hdr, sizeof(CHUNK_HDR), 1, fp) != 1)
			_Error("Error reading chunk header at index %ld in file %s", fpos, filespec) ;

		if (chunk_hdr.id == 0x20746D66) fmt_info = ReadFormatChunk(fp, filespec) ;
		if (chunk_hdr.id == 0x61746164) audio = ReadDataChunk(fp, filespec, chunk_hdr.size) ;

		fseek(fp, fpos + sizeof(CHUNK_HDR) + (chunk_hdr.size + 1) & ~0x1, SEEK_SET) ;
		}

	if (fmt_info == NULL)	_Error("File %s contains no format chunk", filespec) ;
	if (audio == NULL)	_Error("File %s contains no data chunk", filespec) ;

	audio->sample_rate = fmt_info->sample_rate ;
	fclose(fp) ;
	return audio ;
	}

PRIVATE FMT_INFO *ReadFormatChunk(FILE *fp, char *filespec)
	{
	static FMT_INFO fmt_info ;

	if (fread(&fmt_info, sizeof(FMT_INFO), 1, fp) != 1)
		_Error("Can't read format info of file %s", filespec) ;

	if (fmt_info.format != 1)
		_Error("File %s contains incorrect format (%d)", filespec, fmt_info.format) ;

	if (fmt_info.num_channels != 1)
		_Error("File %s contains incorrect number of channels (%d)", filespec, fmt_info.num_channels) ;

	if (fmt_info.bits_per_sample != 8 * sizeof(SAMPLE))
		_Error("File %s contains incorrect bits per sample (%d)", filespec, fmt_info.bits_per_sample) ;

	return &fmt_info ;
	}

PRIVATE AUDIO *ReadDataChunk(FILE *fp, char *filespec, unsigned chunk_size)
	{
	unsigned num_samples, index ;
	AUDIO *audio ;

	num_samples = (unsigned) (chunk_size / sizeof(SAMPLE)) ;
	audio = (AUDIO *) malloc(sizeof(AUDIO) + num_samples * sizeof(SAMPLE)) ;
	if (audio == NULL) _Error("Insufficient memory to load audio file %s", filespec) ;

	audio->num_samples = num_samples ;

	for (index = 0; index < num_samples; index++)
		{
		if (fread(&audio->samples[index], sizeof(SAMPLE), 1, fp) != 1)
			_Error("Can't read audio data from file %s", filespec) ;
		}

	_AddPointer(audio) ;
	return audio ;
	}
